package noown.domain.xsc;

import noown.domain.xsc.val.XSCNameValue;
import noown.domain.xsc.val.XSCValue;

/**
 * Interface for Components that can deal with XML Schema Compact Syntax docs.
 * 
 * Langbeschreibung
 * 
 * @author Kilian Stillhard
 * @version 0.01
 */
public interface XSCHandler
{
	/**
	 * Starts a new component inside an existing one.
	 */
	void startComponent(int type, XSCNameValue name) throws XSCException;
	
	/**
	 * Ends a component
	 */
	void endComponent() throws XSCException;
	
	/**
	 * Adds an extension to the the current component
	 */
	void extension(int type, XSCValue value) throws XSCException;
	
	/**
	 * Adds a qualifier to the next component
	 */
	void qualifier(int type) throws XSCException;
	
	/**
	 * Starts a new Block that contains a list of components
	 */
	void startBlock() throws XSCException;
	
	/**
	 * Ends a Block
	 */
	void endBlock() throws XSCException;
	
	/**
	 * Attaches an annotation to the next component
	 */
	void annotation(String text) throws XSCException;
}
