package noown.domain.xsc.comp;

import java.util.Vector;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCNameValue;
import noown.domain.xsc.val.XSCStringValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Attribute component
 * 
 * @author Kilian Stillhard
 */
public class XSCAttribute extends XSCComponent
{
	private XSCComponent simpleType;
	private Vector options;
	private XSCNameValue name;
	private XSCStringValue fixedDefault;
	private boolean isFixed;
	private boolean isGlobal;
	private XSCAnnotationValue annot;

	public XSCAttribute() throws XSCException
	{
		super();
		isFixed = false;
		options = new Vector();
		isGlobal = XSCDOMBuilder.getBuilder().isGlobal();
	}

	public XSCAttribute(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.ATTRIBUTE;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.NAME_VALUE:
			{ name = (XSCNameValue)value; break; }
			case XSCValue.STRING_VALUE:
			{
				switch (subtype)
				{
					case XSCParserConstants.EQUAL:
					{ fixedDefault = (XSCStringValue)value; isFixed=true; break; }
					case XSCParserConstants.SET_DEF:
					{ fixedDefault = (XSCStringValue)value; isFixed=false; break; }
					default:
						throw new XSCException(XSCException.IllegalProperty,value);
				}
				break;
			}
			case XSCValue.KEYWORD_VALUE:
			{
				switch (((XSCKeywordValue)value).getKey())
				{
					case XSCParserConstants.OPTIONAL:
					case XSCParserConstants.REQUIRED:
					case XSCParserConstants.PROHIBITED:
					case XSCParserConstants.QUALIFIED:
					case XSCParserConstants.UNQUALIFIED:
					{
						if (!isGlobal)
							options.add(value);
						else
							throw new XSCException(XSCException.IllQGlobalAttr,value);
						break;
					}
					default:
						throw new XSCException(XSCException.IllQAttr,value);
				}
				break;
			}
			case XSCValue.RESTRICTION:
			case XSCValue.UNION:
			case XSCValue.LIST:
			{ simpleType = (XSCComponent)value; break;
			}
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value); break; }
			default:
				throw new XSCException(XSCException.IllegalProperty,value);
		}
	}

	protected void freeze_() throws XSCException
	{
		Element el = XSCDOMBuilder.getBuilder().createSchemaElement("attribute");
		
		XSCCommon.addAnnotation(el,annot);
		
		if (simpleType == null && !isGlobal)
			el.setAttribute("ref",name.getXMLString());
		else
			el.setAttribute("name",name.getXMLString());
		
		String form = XSCCommon.getFormAttr(options);
		if (form != null)
			el.setAttribute("form",form);
			
		String use = XSCCommon.getUseAttr(options);
		if (use != null)
			el.setAttribute("use",use);
			
		if (fixedDefault != null && isFixed)
			el.setAttribute("fixed",fixedDefault.getXMLString());
		if (fixedDefault != null && !isFixed)
			el.setAttribute("default",fixedDefault.getXMLString());

		if (simpleType != null && simpleType.getType() == XSCValue.RESTRICTION &&
			((XSCRestriction)simpleType).isTypeRef())
		{
			el.setAttribute("type",((XSCRestriction)simpleType).getTypeRef().getXMLString());
		} else if (simpleType != null) {
				
				XSCSimpleType st = new XSCSimpleType();
				st.property(simpleType,0);
				st.freeze();
				addNodes(el,st.getNodes());
		}
		((XSCNodeListImpl)fNodes).add(el);
	}
}
