package noown.domain.xsc.comp;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Attr;
import org.w3c.dom.NodeList;

/**
 * AttributeDefault components
 *
 * @author Kilian Stillhard
 */
public class XSCAttributeDefault extends XSCComponent
{
	private boolean fQualified;
	private XSCAnnotationValue annot;

	public XSCAttributeDefault() throws XSCException
	{
		super();
		fQualified = false;
	}

	public XSCAttributeDefault(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.DEFAULT_ATTR;
	}

	protected void property_(XSCValue value,int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.KEYWORD_VALUE:
			{
				if (((XSCKeywordValue)value).getKey() == XSCParserConstants.QUALIFIED)
					fQualified = true;
				else if (((XSCKeywordValue)value).getKey() != XSCParserConstants.UNQUALIFIED)
					throw new XSCException(XSCException.IllQAttrDefault,value);
				break;
			}
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value); break; }
			default:
				throw new XSCException(XSCException.IllegalProperty,value);
		}
	}

	protected void freeze_() throws XSCException
	{
		if (annot != null) XSCDOMBuilder.getBuilder().getSchema().property(annot,0);
		
		if (fQualified)
		{
			Attr attr = XSCDOMBuilder.getBuilder().createSchemaAttr("attributeFormDefault");
			attr.setValue("qualified");
			((XSCNodeListImpl)fNodes).add(attr);
		}
	}
}
