package noown.domain.xsc.comp;

import java.util.Vector;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCNameValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Attribute Group component
 * 
 * @author Kilian Stillhard
 */
public class XSCAttributeGroup extends XSCComponent
{
	private XSCNameValue name;
	private boolean isGlobal;
	private Vector contents;
	private XSCAttributeWildcard attWC;
	private XSCAnnotationValue annot;
	
	public XSCAttributeGroup() throws XSCException
	{
		super();
		isGlobal = XSCDOMBuilder.getBuilder().isGlobal();
		contents = new Vector();
	}
	
	public XSCAttributeGroup(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.ATTRIBUTE_GROUP;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.NAME_VALUE:
			{ name = (XSCNameValue)value; break; }
			case XSCValue.ATTRIBUTE:
			case XSCValue.ATTRIBUTE_GROUP:
			{ contents.add(value); break; }
			case XSCValue.ATTRIBUTE_WC:
			{
				if (attWC == null) attWC = (XSCAttributeWildcard)value;
				else throw new XSCException(XSCException.DupAttWC);
				break;
			}
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value); break;}
			default: throw new XSCException(XSCException.IllegalProperty,value);
		}
	}

	protected void freeze_() throws XSCException
	{
		Element ag = XSCDOMBuilder.getBuilder().createSchemaElement("attributeGroup");
		
		XSCCommon.addAnnotation(ag,annot);
		
		if (!isGlobal && (contents.size() > 0 || attWC != null))
			throw new XSCException(XSCException.AttrsInLocalAG);
		
		if (isGlobal)
			ag.setAttribute("name",name.getXMLString());
		else
			ag.setAttribute("ref",name.getXMLString());
		
		if (isGlobal)
		{
			for (int i = 0; i < contents.size(); i++)
				addNodes(ag,((XSCComponent)contents.elementAt(i)).getNodes());
			if (attWC != null)
				addNodes(ag,attWC.getNodes());
		}
		((XSCNodeListImpl)fNodes).add(ag);
	}
}
