package noown.domain.xsc.comp;

import java.util.Vector;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCOccSpecValue;
import noown.domain.xsc.val.XSCStringValue;

/**
 * Some common methods for all components
 * 
 * @author Kilian Stillhard
 */
public class XSCCommon
{

	public static String getFinalAttr(Vector keywords)
	{
		boolean final_all   = false;
		boolean final_restr = false;
		boolean final_ext   = false;
		boolean final_union = false;
		boolean final_list  = false;
		
		if (keywords == null || keywords.size() == 0)
			return null;
		
		for (int i = 0; i < keywords.size();i++)
		{
			switch (((XSCKeywordValue)keywords.elementAt(i)).getKey())
			{
				case XSCParserConstants.FINAL:      { final_all   = true; break; }
				case XSCParserConstants.FINAL_EXT:  { final_ext   = true; break; }
				case XSCParserConstants.FINAL_RESTR:{ final_restr = true; break; }
				case XSCParserConstants.FINAL_LIST: { final_list  = true; break; }
				case XSCParserConstants.FINAL_UNION:{ final_union = true; break; }
				default: break;
			}
		}
		
		if (final_all || final_ext || final_restr || final_union || final_list)
		{
			StringBuffer buf = new StringBuffer();
			boolean hadValue = false;
			if (final_all)
			{
				buf.append("#all");
			} else {
				if (final_ext)
				{
					buf.append("extension");
					hadValue = true;
				}
				if (final_restr)
				{
					if (hadValue) buf.append(" ");
					buf.append("restriction");
					hadValue = true;
				}
				if (final_list)
				{
					if (hadValue) buf.append(" ");
					buf.append("list");
					hadValue = true;
				}
				if (final_union)
				{
					if (hadValue) buf.append(" ");
					buf.append("union");
				}
			}
			return buf.toString();
		} else {
			return null;
		}
	}
	
	public static String getBlockAttr(Vector keywords)
	{
		boolean block_all   = false;
		boolean block_restr = false;
		boolean block_ext   = false;
		boolean block_subst = false;
		
		if (keywords == null || keywords.size() == 0)
			return null;
		
		for (int i = 0; i < keywords.size();i++)
		{
			switch (((XSCKeywordValue)keywords.elementAt(i)).getKey())
			{
				case XSCParserConstants.BLOCK:      { block_all   = true; break; }
				case XSCParserConstants.BLOCK_EXT:  { block_ext   = true; break; }
				case XSCParserConstants.BLOCK_RESTR:{ block_restr = true; break; }
				case XSCParserConstants.BLOCK_SUBST:{ block_subst = true; break; }
				default: break;
			}
		}
		
		if (block_all || block_ext || block_restr || block_subst)
		{
			StringBuffer buf = new StringBuffer();
			boolean hadValue = false;
			if (block_all)
			{
				buf.append("#all");
			} else {
				if (block_ext)
				{
					buf.append("extension");
					hadValue = true;
				}
				if (block_restr)
				{
					if (hadValue) buf.append(" ");
					buf.append("restriction");
					hadValue = true;
				}
				if (block_subst)
				{
					if (hadValue) buf.append(" ");
					buf.append("substitution");
				}
			}
			return buf.toString();
		} else {
			return null;
		}
	}
	
	public static String getFormAttr(Vector keywords)
	{
		boolean qual = false;
		boolean unqual = false;
		
		if (keywords == null || keywords.size() == 0)
			return null;
			
		for (int i = 0; i < keywords.size(); i++)
		{
			switch (((XSCKeywordValue)keywords.elementAt(i)).getKey())
			{
				case XSCParserConstants.QUALIFIED:   { qual = true; break; }
				case XSCParserConstants.UNQUALIFIED: { unqual = true; break; }
				default: break;
			}
		}
		
		if (qual)
			return "qualified";
		else if (unqual)
			return "unqualified";
		else
			return null;
	}
	
	public static String getAbstractAttr(Vector keywords)
	{
		boolean abstr = false;
		
		if (keywords == null || keywords.size() == 0)
			return null;
			
		for (int i = 0; i < keywords.size(); i++)
		{
			switch (((XSCKeywordValue)keywords.elementAt(i)).getKey())
			{
				case XSCParserConstants.ABSTRACT:   { abstr = true; break; }
				default: break;
			}
		}
		
		if (abstr)
			return "true";
		else
			return null;
	}
	
	public static String getNillableAttr(Vector keywords)
	{
		boolean nill = false;
		
		if (keywords == null || keywords.size() == 0)
			return null;
			
		for (int i = 0; i < keywords.size(); i++)
		{
			switch (((XSCKeywordValue)keywords.elementAt(i)).getKey())
			{
				case XSCParserConstants.NILLABLE:   { nill = true; break; }
				default: break;
			}
		}
		
		if (nill)
			return "true";
		else
			return null;
	}
	
	public static String getUseAttr(Vector keywords) throws XSCException
	{
		boolean req = false;
		boolean opt = false;
		boolean proh = false;
		
		if (keywords == null || keywords.size() == 0)
			return null;
			
		for (int i = 0; i < keywords.size(); i++)
		{
			switch (((XSCKeywordValue)keywords.elementAt(i)).getKey())
			{
				case XSCParserConstants.PROHIBITED:   { proh = true; break; }
				case XSCParserConstants.OPTIONAL: { opt = true; break; }
				case XSCParserConstants.REQUIRED: { req = true; break; }
				default: break;
			}
		}
		
		if (!proh && !opt && !req)
			return null;
		if (proh && !opt && !req)
			return "prohibited";
		if (opt && !proh && !req)
			return "optional";
		if (req && !opt && !proh)
			return "required";
			
		throw new XSCException(XSCException.WrongUseQual,keywords.toString());
	}
	
	public static NodeList getOccSpecAttrs(XSCOccSpecValue occSpec)
	{
		XSCNodeListImpl nodes = new XSCNodeListImpl();
		
		String minStr = occSpec.getMinString();
		String maxStr = occSpec.getMaxString();
		
		if (minStr != null)
		{
			Attr min = XSCDOMBuilder.getBuilder().createSchemaAttr("minOccurs");
			min.setValue(minStr);
			nodes.add(min);
		}
		
		if (maxStr != null)
		{
			Attr max = XSCDOMBuilder.getBuilder().createSchemaAttr("maxOccurs");
			max.setValue(maxStr);
			nodes.add(max);
		}
				
		return nodes;
	}

	public static XSCAnnotationValue mergeAnnotations(
		XSCAnnotationValue oldAnnot,
		XSCAnnotationValue newAnnot)
	{
		if (oldAnnot == null)
			return newAnnot;
		else
			return oldAnnot.addAnnot(newAnnot);
	}

	public static void addAnnotation(Element el, XSCAnnotationValue annot) throws XSCException
	{
		if (annot == null) return;
		
		Element an = XSCDOMBuilder.getBuilder().createSchemaElement("annotation");
		Element dc = XSCDOMBuilder.getBuilder().createSchemaElement("documentation");
		Text tx = XSCDOMBuilder.getBuilder().getDoc().createTextNode(annot.getXMLString());
		dc.appendChild(tx);
		an.appendChild(dc);
		el.insertBefore(an,el.getFirstChild());
	}
	
	public static void addAnnotationApp(Element el, XSCAnnotationValue annot) throws XSCException
	{
		if (annot == null) return;
		
		Element an = XSCDOMBuilder.getBuilder().createSchemaElement("annotation");
		Element dc = XSCDOMBuilder.getBuilder().createSchemaElement("documentation");
		Text tx = XSCDOMBuilder.getBuilder().getDoc().createTextNode(annot.getXMLString());
		dc.appendChild(tx);
		an.appendChild(dc);
		el.appendChild(an);
	}

}
