package noown.domain.xsc.comp;

import java.util.Hashtable;
import java.util.Vector;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCNameValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Complex Type component
 * 
 * @author Kilian Stillhard
 */
public class XSCComplexType extends XSCComponent
{
	private XSCNameValue name;
	private XSCNameValue ext;
	private XSCNameValue restr;
	
	private XSCContentModel model;
	private XSCComponent simpleType;
	private Hashtable elements;
	private Vector attributes;
	private XSCAttributeWildcard attWC;
	private Vector options;
	private XSCAnnotationValue annot;

	public XSCComplexType() throws XSCException
	{
		super();
		elements = new Hashtable();
		attributes = new Vector();
		options = new Vector();
	}

	public XSCComplexType(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.COMPLEX_TYPE;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.KEYWORD_VALUE:
			{
				switch (((XSCKeywordValue)value).getKey())
				{
					case XSCParserConstants.ABSTRACT:
					case XSCParserConstants.FINAL:
					case XSCParserConstants.FINAL_EXT:
					case XSCParserConstants.FINAL_RESTR:
					case XSCParserConstants.BLOCK:
					case XSCParserConstants.BLOCK_EXT:
					case XSCParserConstants.BLOCK_RESTR:
					{ options.add(value); break; }
					default:
						throw new XSCException(XSCException.IllQComplexType,value);
				}
				break;
			}
			case XSCValue.NAME_VALUE:
			{
				switch (subtype)
				{
					case 0:
					{ name = (XSCNameValue)value; break; }
					case XSCParserConstants.RESTRICTS:
					{ restr = (XSCNameValue)value; break; }
					case XSCParserConstants.EXTENDS:
					{ ext = (XSCNameValue)value; break; }
					default:break;
				}
				break;
			}
			case XSCValue.RESTRICTION:
			case XSCValue.UNION:
			case XSCValue.LIST:
			{
				if (simpleType != null) throw new XSCException(XSCException.DupSTinCT);
				if (model != null) throw new XSCException(XSCException.DupTypeDefInCT);
				simpleType = (XSCComponent)value;
				break;
			}
			case XSCValue.CONTENT_MODEL:
			{
				if (simpleType != null) throw new XSCException(XSCException.DupTypeDefInCT);
				if (model != null) throw new XSCException(XSCException.DupCMinCT);
				model = (XSCContentModel)value;
				break;
			}
			case XSCValue.ELEMENT:
			{ 
				if (! elements.containsKey(((XSCElement)value).getName()))
					elements.put(((XSCElement)value).getName(), value);
				else
					throw new XSCException(XSCException.DupElementInCT,((XSCElement)value).getName()); 
				break;
			}
			case XSCValue.ATTRIBUTE:
			case XSCValue.ATTRIBUTE_GROUP:
			{ attributes.add(value); break; }
			case XSCValue.ATTRIBUTE_WC:
			{
				if (attWC != null) throw new XSCException(XSCException.DupAttWCinCT);
				attWC = (XSCAttributeWildcard)value;
				break;
			}
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)annot); break;}
			default:
				throw new XSCException(XSCException.IllegalProperty,value);
			
		}
	}

	protected void freeze_() throws XSCException
	{
		Element ct = XSCDOMBuilder.getBuilder().createSchemaElement("complexType");
		XSCCommon.addAnnotation(ct,annot);
		if (name != null) ct.setAttribute("name",name.getXMLString());
		
		// the element that will hold the content model / simple type
		Element typeDef = ct;
		
		// check for simple content
		if (simpleType != null)
		{
			Element sc = XSCDOMBuilder.getBuilder().createSchemaElement("simpleContent");
			ct.appendChild(sc);
			
			if (simpleType.getType() != XSCValue.RESTRICTION)
				throw new XSCException(XSCException.ListUnionInCT);
				
			if (ext != null || restr != null)
				throw new XSCException(XSCException.DerAndSTinCT);
				
			// check if extension or restriction
			if (((XSCRestriction)simpleType).isTypeRef())
			{
				Element ex = XSCDOMBuilder.getBuilder().createSchemaElement("extension");
				sc.appendChild(ex);
				typeDef = ex;
				
				XSCNameValue base = ((XSCRestriction)simpleType).getTypeRef();
				ex.setAttribute("base",base.getXMLString());
			} else {
				Element re = (Element) simpleType.getNodes().item(0);
				addNodes(sc,simpleType.getNodes());
				typeDef = re;
			}
		} else {
			
			if (model != null && model.isMixed()) ct.setAttribute("mixed","true");
			
			if (ext != null || restr != null)
			{
				Element cc = XSCDOMBuilder.getBuilder().createSchemaElement("complexContent");
				ct.appendChild(cc);
				if (ext != null)
				{
					Element ex = XSCDOMBuilder.getBuilder().createSchemaElement("extension");
					cc.appendChild(ex);
					ex.setAttribute("base",ext.getXMLString());
					typeDef = ex;
				} else {
					Element re = XSCDOMBuilder.getBuilder().createSchemaElement("restriction");
					cc.appendChild(re);
					re.setAttribute("base",restr.getXMLString());
					typeDef = re;
				}
			}
			
			if (model != null)
			{	
				model.addElementDecls(elements);
				addNodes(typeDef,model.getNodes());
			}
		}
		
		for (int i = 0; i < attributes.size(); i++)
			addNodes(typeDef,((XSCComponent)attributes.elementAt(i)).getNodes());
			
		if (attWC != null)
			addNodes(typeDef,attWC.getNodes());
		
		((XSCNodeListImpl)fNodes).add(ct);
	}
}
