package noown.domain.xsc.comp;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCValue;

/**
 * Kurzbeschreibung
 * 
 * Langbeschreibung
 * 
 * @author Kilian Stillhard
 * @version 0.01
 */
public abstract class XSCComponent extends XSCValue
{
	protected NodeList fNodes;
	
	protected boolean fFrozen;

	public XSCComponent() throws XSCException
	{
		fNodes = null;
		fFrozen = false;
	}
	
	public XSCComponent(NodeList nodes) throws XSCException
	{
		fNodes = nodes;
		fFrozen = true;
	}
	
	public NodeList getNodes() throws XSCException
	{
		if ((fNodes != null) && fFrozen)
			return fNodes;
		else
			throw new XSCException("getNodes() called while component not frozen!");
	}
	
	public String getXSCString() throws XSCException
	{
		throw new XSCException("getXSCString() not supported for this component!");
	}
	
	public String getString() throws XSCException
	{
		throw new XSCException("getString() not supported for this component!");
	}
	
	abstract public int getType();
	
	public void property(XSCValue value,int subtype) throws XSCException
	{
		if (fFrozen)
			throw new XSCException("property() called while component frozen!");
		else
			property_(value,subtype);
	}
	
	abstract protected void property_(XSCValue value,int subtype) throws XSCException;
	
	public void freeze() throws XSCException
	{
		fFrozen = true;
		if (fNodes == null) fNodes = new XSCNodeListImpl(); 
		freeze_();
	}
	
	abstract protected void freeze_() throws XSCException;
	
	protected void addNodes(Element el, NodeList nodes)
	{
		for (int i = 0; i < nodes.getLength(); i++)
		{
			switch (nodes.item(i).getNodeType())
			{
				case Node.ATTRIBUTE_NODE:
				{
					el.setAttributeNode((Attr)nodes.item(i));
					break;
				}
				
				case Node.ELEMENT_NODE:
				{
					el.appendChild(nodes.item(i));
					break;
				}
				
				default: break;
			}
		}
	}

}
