package noown.domain.xsc.comp;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCOccSpecValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Content model component
 * 
 * @author Kilian Stillhard
 */
public class XSCContentModel extends XSCComponent
{
	private XSCGroup group;
	private XSCModelGroup model;
	private boolean mixed;
	private XSCAnnotationValue annot;
	
	public XSCContentModel() throws XSCException
	{
		super();
	}

	public XSCContentModel(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.CONTENT_MODEL;
	}

	public boolean isMixed()
	{
		return mixed;
	}
	
	public void addElementDecls(Hashtable elements) throws XSCException
	{
		Hashtable els = (Hashtable)elements.clone();
		
		if (!fFrozen)
			throw new XSCException("addElementDecls() while content model not frozen!");
			
		Element top = (Element)fNodes.item(0);
		
		if (top == null || top.getLocalName().equals("group") || els.size() == 0)
			return;
			
		addElementDecls_(top,els);
		
		for (Enumeration e = els.keys(); e.hasMoreElements();)
			throw new XSCException(XSCException.ElNotInCM,((XSCElement)els.get(e.nextElement())).getName());
	}

	private void addElementDecls_(Element top, Hashtable els) throws XSCException
	{
		Node n = top.getFirstChild();
		
		while (n != null)
		{
			if (n.getNodeType() == Node.ELEMENT_NODE &&
			   (n.getLocalName().equals("sequence") ||
			    n.getLocalName().equals("choice")))
			    
			    addElementDecls_((Element)n,els);
		
			if (n.getNodeType() == Node.ELEMENT_NODE &&
			    n.getLocalName().equals("element") &&
			    ((Element)n).hasAttribute("ref"))
			{
				String ref = ((Element)n).getAttribute("ref");
				XSCElement def = (XSCElement)els.get(ref);
				if (def != null)
				{
					// a element with ref="" attribute can have only minOccurs
					// and maxOccurs attributes (if generated by us)
					String minOcc = ((Element)n).getAttribute("minOccurs");
					String maxOcc = ((Element)n).getAttribute("maxOccurs");
					Element defEl = (Element)def.getNodes().item(0);
					if (minOcc.length() > 0) defEl.setAttribute("minOccurs",minOcc);
					if (maxOcc.length() > 0) defEl.setAttribute("maxOccurs",maxOcc);
					top.replaceChild(defEl,n);
					els.remove(ref);
					n = defEl;
				}
			}
			n = n.getNextSibling();
		}
	}


	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.KEYWORD_VALUE:
			{
				switch (((XSCKeywordValue)value).getKey())
				{
					case XSCParserConstants.MIXED:
					{ mixed = true; break; }
					case XSCParserConstants.EMPTY: break;
					default:break;
				}
				break;
			}
			case XSCValue.GROUP:
			{ group = (XSCGroup)value; break; }
			case XSCValue.MODEL_GROUP:
			{ model = (XSCModelGroup)value; break; }
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value); break;}
			default: break;
		}
	}

	protected void freeze_() throws XSCException
	{
		if (group != null)
		{
			((XSCNodeListImpl)fNodes).add(group.getNodes().item(0));
		} else if (model != null) {
			((XSCNodeListImpl)fNodes).add(model.getNodes().item(0));
		}
		
		Element cm = (Element) fNodes.item(0);
		if (cm != null)
			XSCCommon.addAnnotation(cm,annot);
	}

}
