package noown.domain.xsc.comp;

import java.util.Vector;

import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCNameValue;
import noown.domain.xsc.val.XSCOccSpecValue;
import noown.domain.xsc.val.XSCStringValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

/**
 * Kurzbeschreibung
 * 
 * Langbeschreibung
 * 
 * @author Kilian Stillhard
 * @version 0.01
 */
public class XSCElement extends XSCComponent
{
	private XSCNameValue name;
	private XSCNameValue subst;
	private XSCNameValue restr;
	private XSCNameValue ext;
	
	private XSCStringValue fixedDefault;
	private boolean isFixed;
	private boolean isGlobal;
	private boolean inContentModel;
	
	private Vector options;
	
	private XSCContentModel model;
	private XSCComponent simpleType;
	private XSCOccSpecValue occSpec;
	private Vector elements;
	private Vector attributes;
	private Vector keys;
	private XSCAnnotationValue annot;
	
	public XSCElement() throws XSCException
	{
		super();
		options = new Vector();
		elements = new Vector();
		attributes = new Vector();
		keys = new Vector();
		isFixed = false;
		inContentModel = false;
		isGlobal = XSCDOMBuilder.getBuilder().isGlobal();
	}
	public XSCElement(NodeList nodes) throws XSCException
	{
		super(nodes);
	}
	
	public int getType()
	{
		return XSCValue.ELEMENT;
	}
	
	public String getName() throws XSCException
	{
		if (name != null && fFrozen)
			return name.getXMLString();
		else
			throw new XSCException("Cannot call getName()");
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.NAME_VALUE:
			{
				switch (subtype)
				{
					case 0:
					{ name = (XSCNameValue)value; break; }
					case XSCParserConstants.SUBSTITUTES:
					{ subst = (XSCNameValue)value; break; }
					case XSCParserConstants.EXTENDS:
					{ ext = (XSCNameValue)value; break; }
					case XSCParserConstants.RESTRICTS:
					{ restr = (XSCNameValue)value; break; }
					default:
						throw new XSCException("Illegal property");
				}
				break;
			}
			
			case XSCValue.KEYWORD_VALUE:
			{ 
				switch (((XSCKeywordValue)value).getKey())
				{
					case XSCParserConstants.FINAL:
					case XSCParserConstants.FINAL_EXT:
					case XSCParserConstants.FINAL_RESTR:
					{ 
						if (isGlobal) options.add(value);
						else throw new XSCException("final qualifier in local element!");
						break;
					}  
					case XSCParserConstants.BLOCK:
					case XSCParserConstants.BLOCK_EXT:
					case XSCParserConstants.BLOCK_RESTR:
					case XSCParserConstants.BLOCK_SUBST:
					{ options.add(value); break; }
					case XSCParserConstants.ABSTRACT:
					{
						if (isGlobal) options.add(value);
						else throw new XSCException("abstract qualifier in local element!");
						break;
					}
					case XSCParserConstants.NILLABLE:
					{ options.add(value); break; }
					case XSCParserConstants.QUALIFIED:
					case XSCParserConstants.UNQUALIFIED:
					{
						if (!isGlobal) options.add(value);
						else throw new XSCException("form qualifier in global element!");
						break;
					}
					case XSCParserConstants.OPEN_PAR:
					{ inContentModel = true; break; }
					default:
						throw new XSCException("illegal qualifier for element!");
				}
				break;
			}
			case XSCValue.STRING_VALUE:
			{	
				switch (subtype)
				{
					case XSCParserConstants.EQUAL:
					{
						if (!isGlobal) { fixedDefault=(XSCStringValue)value;isFixed=true; }
						else throw new XSCException("fixed declaration on global element!");
						break;
					}
					case XSCParserConstants.SET_DEF:
					{
						if (!isGlobal) { fixedDefault=(XSCStringValue)value;isFixed=false; }
						else throw new XSCException("default declaration on global element!");
						break;
					}
					default:
						throw new XSCException("Illegal property");
				}
				break;
			}
			case XSCValue.OCCSPEC_VALUE:
			{ occSpec = (XSCOccSpecValue)value; break; }
			
			case XSCValue.RESTRICTION:
			case XSCValue.UNION:
			case XSCValue.LIST:
			{
				if (simpleType != null || model != null)
					throw new XSCException("Duplicate type definition");
				simpleType = (XSCComponent)value;
				break;
			}
			case XSCValue.CONTENT_MODEL:
			{
				if (simpleType != null || model != null)
					throw new XSCException("Duplicate type definition");
				model = (XSCContentModel)value;
				break;
			}
			case XSCValue.ELEMENT:
			{
				if (simpleType != null)
					throw new XSCException("Local elements but simple content!");
				elements.add(value);
				break;
			}
			case XSCValue.ATTRIBUTE:
			case XSCValue.ATTRIBUTE_GROUP:
			case XSCValue.ATTRIBUTE_WC:
			{
				attributes.add(value);
				break;
			}
			
			case XSCValue.KEY:
			case XSCValue.KEYREF:
			case XSCValue.UNIQUE:
			{
				keys.add(value);
				break;
			}
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value); break;}
			default:
				throw new XSCException("Illegal property");	
		}
				
	}

	protected void freeze_() throws XSCException
	{
		Element el = XSCDOMBuilder.getBuilder().createSchemaElement("element");
		XSCCommon.addAnnotation(el,annot);
		
		// check if there is a complex type def
		if (ext != null || restr != null || model != null || attributes.size() > 0
			|| elements.size() > 0 )
		{
			el.setAttribute("name",name.getXMLString());
			
			XSCComplexType ct = new XSCComplexType();
			
			if (model != null) ct.property(model,0);
			if (simpleType != null) ct.property(simpleType,0);
			if (ext != null) ct.property(ext,XSCParserConstants.EXTENDS);
			if (restr != null) ct.property(restr,XSCParserConstants.RESTRICTS);
			
			for (int i = 0; i < attributes.size(); i++)
				ct.property((XSCComponent)attributes.elementAt(i),0);
			for (int i = 0; i < elements.size(); i++)
				ct.property((XSCComponent)elements.elementAt(i),0);
			
			ct.freeze();
			addNodes(el,ct.getNodes());
			
		} else if (simpleType != null) {
			// check if there is a simple type def
			el.setAttribute("name",name.getXMLString());
			
			if (simpleType.getType() == XSCValue.RESTRICTION &&
				((XSCRestriction)simpleType).isTypeRef())
			{
				el.setAttribute("type",((XSCRestriction)simpleType).getTypeRef().getXMLString());
			} else {
				
				XSCSimpleType st = new XSCSimpleType();
				st.property(simpleType,0);
				st.freeze();
				addNodes(el,st.getNodes());
			}
		} else if (keys.size() == 0 && inContentModel)
		{
			// it is a reference
			el.setAttribute("ref",name.getXMLString());
		} else {
			// other cases
			el.setAttribute("name",name.getXMLString());
		}
		
		if (subst != null)
			el.setAttribute("substitutionGroup",subst.getXMLString());
			
		String fin = XSCCommon.getFinalAttr(options);
		if (fin != null)
			el.setAttribute("final",fin);
			
		String block = XSCCommon.getBlockAttr(options);
		if (block != null)
			el.setAttribute("block",block);
			
		String nill = XSCCommon.getNillableAttr(options);
		if (nill != null)
			el.setAttribute("nillable",nill);
			
		String abstr = XSCCommon.getAbstractAttr(options);
		if (abstr != null)
			el.setAttribute("abstract",abstr);
			
		String form = XSCCommon.getFormAttr(options);
		if (form != null)
			el.setAttribute("form",form);
			
		if (fixedDefault != null && isFixed)
			el.setAttribute("fixed",fixedDefault.getXMLString());
		if (fixedDefault != null && !isFixed)
			el.setAttribute("default",fixedDefault.getXMLString());
			
		if (occSpec != null)
			addNodes(el,XSCCommon.getOccSpecAttrs(occSpec));
		
		for (int i = 0; i < keys.size();i++)
			addNodes(el,((XSCComponent)keys.elementAt(i)).getNodes());
			
		((XSCNodeListImpl)fNodes).add(el);
	}
}
