package noown.domain.xsc.comp;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Attr;
import org.w3c.dom.NodeList;

/**
 * Kurzbeschreibung
 * 
 * Langbeschreibung
 * 
 * @author Kilian Stillhard
 * @version 0.01
 */
public class XSCElementDefault extends XSCComponent
{
	private boolean fQualified;
	private XSCAnnotationValue annot;
	
	public XSCElementDefault() throws XSCException
	{
		super();
		// The default value is true
		fQualified = true;
	}

	public XSCElementDefault(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.DEFAULT_EL;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.KEYWORD_VALUE:
			{
				if (((XSCKeywordValue)value).getKey() == XSCParserConstants.UNQUALIFIED)
					fQualified = false;
				else if (((XSCKeywordValue)value).getKey() != XSCParserConstants.QUALIFIED)
					throw new XSCException(XSCException.IllQElDefault,value);
				break;
			}
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value); break; }
			default:
				throw new XSCException(XSCException.IllegalProperty,value);
		}
	}

	protected void freeze_() throws XSCException
	{
		if (annot != null) XSCDOMBuilder.getBuilder().getSchema().property(annot,0);
		
		if (fQualified)
		{
			Attr attr = XSCDOMBuilder.getBuilder().createSchemaAttr("elementFormDefault");
			attr.setValue("qualified");
			((XSCNodeListImpl)fNodes).add(attr);
		}
	}
}
