package noown.domain.xsc.comp;

import java.util.Hashtable;
import java.util.Vector;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCStringValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Enumeration Facet
 * 
 * @author Kilian Stillhard
 */
public class XSCEnumFacet extends XSCFacet
{
	private Vector values;
	private Hashtable annots;
	
	public XSCEnumFacet() throws XSCException
	{
		super();
		values = new Vector();
		annots = new Hashtable();
	}

	public XSCEnumFacet(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.ENUM_FACET;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		super.property_(value,subtype);
		switch (value.getType())
		{
			case XSCValue.STRING_VALUE:
			{ values.add(value); break; }
			case XSCValue.ANNOT_VALUE:
			{ annots.put(new Integer(values.size()),value); break; }
			// cannot throw exception here
			default: break;
		}
	}

	protected void freeze_() throws XSCException
	{
		Element enum = null;
		XSCAnnotationValue an = null;
		
		for (int i = 0; i < values.size(); i++)
		{
			enum = XSCDOMBuilder.getBuilder().createSchemaElement("enumeration");
			
			if ((an = (XSCAnnotationValue)annots.get(new Integer(i))) != null)
				XSCCommon.addAnnotation(enum,an);
			enum.setAttribute("value",((XSCStringValue)values.elementAt(i)).getXMLString());
			((XSCNodeListImpl)fNodes).add(enum);
		}
		
		if (isFixed() || isFixedMax() || isFixedMin())
			throw new XSCException(XSCException.FixedEnum);		
	}
}
