package noown.domain.xsc.comp;

import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCValue;
import org.w3c.dom.NodeList;

/**
 * Superclass for all facets
 * 
 * @author Kilian Stillhard
 */
public abstract class XSCFacet extends XSCComponent
{
	private boolean fixed;
	private boolean fixed_min;
	private boolean fixed_max;
	
	public XSCFacet() throws XSCException
	{
		super();
		fixed = false;
		fixed_min = false;
		fixed_max = false;
	}

	public XSCFacet(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	abstract public int getType();
	
	protected boolean isFixed()
	{
		return fixed;
	}
	
	protected boolean isFixedMin()
	{
		return fixed_min;
	}
	
	protected boolean isFixedMax()
	{
		return fixed_max;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.KEYWORD_VALUE:
			{
				switch (((XSCKeywordValue)value).getKey())
				{
					case XSCParserConstants.FIXED:
					{ fixed = true; break; }
					case XSCParserConstants.FIXED_MIN:
					{ fixed_min = true; break; }
					case XSCParserConstants.FIXED_MAX:
					{ fixed_max = true; break; }
					case XSCParserConstants.PRESERVE:
					case XSCParserConstants.COLLAPSE:
					case XSCParserConstants.REPLACE:
						break;
					default:
						throw new XSCException(XSCException.IllegalProperty);
				}
				break;
			}
			default: break;
		}
	}

	abstract protected void freeze_() throws XSCException;

}
