package noown.domain.xsc.comp;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCIntValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Fraction Digits facet
 * 
 * @author Kilian Stillhard
 */
public class XSCFractionDigitsFacet extends XSCFacet
{
	private XSCIntValue value;
	private XSCAnnotationValue annot;
	
	public XSCFractionDigitsFacet() throws XSCException
	{
		super();
	}

	public XSCFractionDigitsFacet(NodeList nodes) throws XSCException
	{
		super(nodes);
	}	

	public int getType()
	{
		return XSCValue.FRACDIG_FACET;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		super.property_(value,subtype);
		switch (value.getType())
		{
			case XSCValue.INTEGER_VALUE:
			{ value = (XSCIntValue)value; break; }
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value); break;}
			default: break;
		}
	}

	protected void freeze_() throws XSCException
	{
		Element el = XSCDOMBuilder.getBuilder().createSchemaElement("fractionDigits");
		XSCCommon.addAnnotation(el,annot);				
		if (value != null)
			el.setAttribute("value",value.getXMLString());
		
		if (isFixed())
			el.setAttribute("fixed","true");
			
		if (isFixedMin() || isFixedMax())
			throw new XSCException(XSCException.FixedFracDig);
			
		((XSCNodeListImpl)fNodes).add(el);
	}
}
