package noown.domain.xsc.comp;

import java.util.Hashtable;
import java.util.Vector;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.XSCWarning;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCNameValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Group component
 * 
 * @author Kilian Stillhard
 */
public class XSCGroup extends XSCComponent
{
	private XSCNameValue name;
	private XSCContentModel model;
	private Hashtable elements;
	private boolean isGlobal;
	private XSCAnnotationValue annot;

	public XSCGroup() throws XSCException
	{
		super();
		elements = new Hashtable();
		isGlobal = XSCDOMBuilder.getBuilder().isGlobal();
	}
	
	public XSCGroup(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.GROUP;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.NAME_VALUE:
			{ name = (XSCNameValue)value; break; }
			case XSCValue.CONTENT_MODEL:
			{ 
				if (model != null) throw new XSCException(XSCException.DupCMinGroup);
				model = (XSCContentModel)value;
				break;
			}
			case XSCValue.ELEMENT:
			{ 
				if (! elements.containsKey(((XSCElement)value).getName()))
					elements.put(((XSCElement)value).getName(), value);
				else
					throw new XSCException(XSCException.DupElInGroup,((XSCElement)value).getName()); 
				break;
			}
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value); break;}
			default: break;
		}
	}

	protected void freeze_() throws XSCException
	{
		Element gr = XSCDOMBuilder.getBuilder().createSchemaElement("group");
		XSCCommon.addAnnotation(gr,annot);
		if (isGlobal)
		{
			gr.setAttribute("name",name.getXMLString());
			if (model != null)
			{
				model.addElementDecls(elements);
				addNodes(gr,model.getNodes());
			} else {
				XSCWarning.add("Empty group: " + name.getXMLString());
				XSCModelGroup emptyModel = new XSCModelGroup();
				emptyModel.property(new XSCKeywordValue(XSCParserConstants.SEQ),0);
				emptyModel.freeze();
				addNodes(gr,emptyModel.getNodes());
			}
		}
		else
			gr.setAttribute("ref",name.getXMLString());
		
		((XSCNodeListImpl)fNodes).add(gr);
	}
}
