package noown.domain.xsc.comp;

import java.util.Vector;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCNameValue;
import noown.domain.xsc.val.XSCValue;
import noown.domain.xsc.val.XSCXPathValue;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Base class for all Identity Constraints
 * 
 * @author Kilian Stillhard
 */
public abstract class XSCIDConstraint extends XSCComponent
{
	private XSCNameValue name;
	private Vector fields;
	private XSCXPathValue selector;
	protected XSCAnnotationValue annot;

	public XSCIDConstraint() throws XSCException
	{
		super();
		fields = new Vector();
	}

	public XSCIDConstraint(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	abstract public int getType();
	
	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.NAME_VALUE:
			{ if (subtype == 0) name = (XSCNameValue)value; break; }
			case XSCValue.XPATH_VALUE:
			{
				if (subtype == XSCParserConstants.FIELD)
					fields.add(value);
				else
					selector = (XSCXPathValue)value;
				break;
			}
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value);break;}
			default:break;
		}
	}
	
	protected NodeList getIDConstr() throws XSCException
	{
		XSCNodeListImpl nodes = new XSCNodeListImpl();
		
		Element se = XSCDOMBuilder.getBuilder().createSchemaElement("selector");
		se.setAttribute("xpath",selector.getXMLString());
		nodes.add(se);
		
		for (int i = 0; i < fields.size(); i++)
		{
			Element fi = XSCDOMBuilder.getBuilder().createSchemaElement("field");
			fi.setAttribute("xpath",((XSCXPathValue)fields.elementAt(i)).getXMLString());
			nodes.add(fi);
		}
		
		Attr nm = XSCDOMBuilder.getBuilder().createSchemaAttr("name");
		nm.setValue(name.getXMLString());
		nodes.add(nm);
		
		return nodes;
	}
	
	abstract protected void freeze_() throws XSCException;

}
