package noown.domain.xsc.comp;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCURIValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Import statement
 * 
 * @author Kilian Stillhard
 */
public class XSCImport extends XSCComponent
{
	private XSCURIValue fURI;
	private XSCURIValue fNS;
	private XSCAnnotationValue annot;

	public XSCImport() throws XSCException
	{
		super();
	}

	public XSCImport(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.IMPORT;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.URI_VALUE:
			{
				if (subtype == XSCParserConstants.IMPORT)
					fURI = (XSCURIValue)value;
				else 
					fNS = (XSCURIValue)value;
				break;
			}
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value); break;}
			default:
				throw new XSCException(XSCException.IllegalProperty,value);
		}
	}

	protected void freeze_() throws XSCException
	{
		Element imp = XSCDOMBuilder.getBuilder().createSchemaElement("import");
		XSCCommon.addAnnotation(imp,annot);
		imp.setAttribute("schemaLocation",fURI.getXMLString());
		imp.setAttribute("namespace",fNS.getXMLString());
		
		((XSCNodeListImpl)fNodes).add(imp);
	}

}
