package noown.domain.xsc.comp;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCIntValue;
import noown.domain.xsc.val.XSCPosIntRange;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Length facet
 * 
 * @author Kilian Stillhard
 */
public class XSCLengthFacet extends XSCFacet
{
	private XSCIntValue val;
	private XSCPosIntRange range;
	private XSCAnnotationValue annot;
	
	public XSCLengthFacet() throws XSCException
	{
		super();
	}
	
	public XSCLengthFacet(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.LENGTH_FACET;
	}
	
	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		super.property_(value,subtype);
		switch (value.getType())
		{
			case XSCValue.INTEGER_VALUE:
			{ val = (XSCIntValue)value; break; }
			case XSCValue.INTRANGE_VALUE:
			{ range = (XSCPosIntRange)value; break; }
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value);break;}
			default: break;
		}
	}
	
	protected void freeze_() throws XSCException
	{
		Element last = null;
		
		if (val != null)
		{
			Element length = XSCDOMBuilder.getBuilder().createSchemaElement("length");
			last = length;
			length.setAttribute("value",val.getXMLString());
			if (isFixed()) length.setAttribute("fixed","true");
			if (isFixedMin() || isFixedMax())
				throw new XSCException(XSCException.FixedLength);
			
			((XSCNodeListImpl)fNodes).add(length);						
		} else if (range != null)
		{
			int min = range.getMin();
			int max = range.getMax();
			
			if (min > -1)
			{
				Element minLength = XSCDOMBuilder.getBuilder().createSchemaElement("minLength");
				last = minLength;
				minLength.setAttribute("value",Integer.toString(min));
				if (isFixed() || isFixedMin()) minLength.setAttribute("fixed","true");
				((XSCNodeListImpl)fNodes).add(minLength);
			}
			if (max > -1)
			{
				Element maxLength = XSCDOMBuilder.getBuilder().createSchemaElement("maxLength");
				last = maxLength;
				maxLength.setAttribute("value",Integer.toString(max));
				if (isFixed() || isFixedMax()) maxLength.setAttribute("fixed","true");
				((XSCNodeListImpl)fNodes).add(maxLength);
			}
		}
		if (last != null) XSCCommon.addAnnotation(last,annot);	
	}

}