package noown.domain.xsc.comp;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * List simple type component
 * 
 * @author Kilian Stillhard
 */
public class XSCList extends XSCComponent
{
	private XSCComponent itemType;
	private XSCAnnotationValue annot;
	
	public XSCList() throws XSCException
	{
		super();
	}
	
	public XSCList(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.LIST;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.UNION:
			case XSCValue.RESTRICTION:
			{ itemType = (XSCComponent)value; break; }
			case XSCValue.LIST:
			{ throw new XSCException(XSCException.ListOfList); }
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value);break;}
			default: break;
		}
	}

	protected void freeze_() throws XSCException
	{
		Element li = XSCDOMBuilder.getBuilder().createSchemaElement("list");
		XSCCommon.addAnnotation(li,annot);
		if (itemType.getType() == XSCValue.RESTRICTION &&
			((XSCRestriction)itemType).isTypeRef())
		{
			li.setAttribute("itemType",((XSCRestriction)itemType).getTypeRef().getXMLString());
		} else {
			XSCSimpleType st = new XSCSimpleType();
			st.property(itemType,0);
			st.freeze();
			addNodes(li,st.getNodes());
		}
		((XSCNodeListImpl)fNodes).add(li);
	}

}
