package noown.domain.xsc.comp;

import java.util.Vector;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCOccSpecValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Model Group Component
 * 
 * @author Kilian Stillhard
 */
public class XSCModelGroup extends XSCComponent
{
	private XSCKeywordValue compositor;
	private Vector particles;
	private XSCOccSpecValue occSpec;

	public XSCModelGroup() throws XSCException
	{
		super();
		particles = new Vector();
	}

	public XSCModelGroup(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.MODEL_GROUP;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.KEYWORD_VALUE:
			{
				switch (((XSCKeywordValue)value).getKey())
				{
					case 0:
					{ compositor = new XSCKeywordValue(XSCParserConstants.SEQ); break; }
					case XSCParserConstants.SEQ:
					case XSCParserConstants.CHOICE:
					case XSCParserConstants.ALL:
					{ compositor = (XSCKeywordValue)value; break; }
					default: break;
				}
				break;
			}
			case XSCValue.ELEMENT:
			case XSCValue.ELEMENT_WC:
			case XSCValue.MODEL_GROUP:
			case XSCValue.GROUP:
			{ particles.add(value); break; }
			case XSCValue.OCCSPEC_VALUE:
			{ occSpec = (XSCOccSpecValue)value; break; }
			case XSCValue.ANNOT_VALUE: break; //ignore annotations
			default:break;
		}
	}

	protected void freeze_() throws XSCException
	{
		Element mg = null;
		if (compositor.getKey() == XSCParserConstants.SEQ)
			mg = XSCDOMBuilder.getBuilder().createSchemaElement("sequence");
		if (compositor.getKey() == XSCParserConstants.CHOICE)
			mg = XSCDOMBuilder.getBuilder().createSchemaElement("choice");
		if (compositor.getKey() == XSCParserConstants.ALL)
			mg = XSCDOMBuilder.getBuilder().createSchemaElement("all");
		
		if (occSpec != null)
			addNodes(mg,XSCCommon.getOccSpecAttrs(occSpec));
			
		for (int i = 0; i < particles.size(); i++)
			addNodes(mg,((XSCComponent)particles.elementAt(i)).getNodes());
			
		((XSCNodeListImpl)fNodes).add(mg);
	}
}
