package noown.domain.xsc.comp;

import java.util.Hashtable;
import java.util.Enumeration;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.XSCWarning;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCNameValue;
import noown.domain.xsc.val.XSCURIValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Attr;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Namespace declaration
 * 
 * @author Kilian Stillhard
 */
public class XSCNamespace extends XSCComponent
{
	protected static Hashtable fNamespaces;
	protected static String fTargetNS;
	protected static String fSchemaPrefix;
	
	public static final String SCHEMA_NS = "http://www.w3.org/2001/XMLSchema";
	public static final String NAMESPACE_NS = "http://www.w3.org/2000/xmlns/";
	public static final String XML_NS = "http://www.w3.org/XML/1998/namespace";
	
	private String fPrefix;
	
	public XSCNamespace() throws XSCException
	{
		super();
		fPrefix = null;
	}
	
	public static void resetNamespaces()
	{
		fNamespaces = new Hashtable();
		fTargetNS = null;
		fSchemaPrefix = null;
	}
	
	public static void addNamespace(String prefix, String uri) throws XSCException
	{	
		if (fNamespaces.get(prefix) != null)
			XSCWarning.add("Duplicate Prefix Definition for " + prefix);
		
		if (uri.equals(SCHEMA_NS)) fSchemaPrefix = prefix;
		
		fNamespaces.put(prefix,uri);
	}
	
	public static void initNamespaces() throws XSCException
	{
		fNamespaces = new Hashtable();
		fTargetNS = null;
		addNamespace("xs",SCHEMA_NS);
	}
	
	public static String getSchemaName(String el)
	{
		if (fSchemaPrefix.equals(""))
			return el;
		else
			return fSchemaPrefix + ":" + el;	
	}
	
	public static boolean hasPrefix(String prefix)
	{
		// these are always known... (probably not the right way to do this)
		if (prefix.startsWith("xml")) return true;
		
		return fNamespaces.containsKey(prefix);
	}
	
	public static String getSchemaPrefix()
	{
		return fSchemaPrefix;
	}
	
	public static void cleanup() throws XSCException
	{
		// if there is a target namespace but no prefix declaration for it,
		// we declare it as the default namespace (unless this is already specified)
		if (fTargetNS != null && (!fNamespaces.containsValue(fTargetNS))
			&& (!fNamespaces.containsKey("")))
			addNamespace("",fTargetNS);
			
		// if another prefix (not xs) was defined for the xml schema namespace,
		// the definition for xs is removed.
		String prefix;
		for (Enumeration e = fNamespaces.keys(); e.hasMoreElements(); )
		{
			prefix = (String)e.nextElement();
			if (((String)fNamespaces.get(prefix)).equals(SCHEMA_NS) &&
			    !prefix.equals("xs"))
			    if (((String)fNamespaces.get("xs")).equals(SCHEMA_NS))
			    	fNamespaces.remove("xs");
		}
	}
		
	
	public static NodeList getNodesNS()
	{
		XSCNodeListImpl nodes = new XSCNodeListImpl();
		
		if (fTargetNS != null)
		{
			Attr tns = XSCDOMBuilder.getBuilder().createSchemaAttr("targetNamespace");
			tns.setValue(fTargetNS);
			nodes.add(tns);
		}
		
		for (Enumeration pref = fNamespaces.keys();pref.hasMoreElements();)
		{
			Attr ns;
			String prefix = (String)pref.nextElement();
			
			if (prefix.equals(""))
				ns = XSCDOMBuilder.getBuilder().getDoc().createAttributeNS(NAMESPACE_NS,"xmlns");
			else
				ns = XSCDOMBuilder.getBuilder().getDoc().createAttributeNS(NAMESPACE_NS,"xmlns:"+prefix);
				
			ns.setValue((String)fNamespaces.get(prefix));
			nodes.add(ns);
		}
		return nodes;		
	}
	
	public static boolean isSchemaElement(Element el)
	{
		return el.getNamespaceURI().equals(SCHEMA_NS);
	}
	
	public int getType()
	{
		return XSCValue.NAMESPACE;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.URI_VALUE:
			{
				String ns = ((XSCURIValue)value).getXMLString();
				if (fPrefix == null) fPrefix = "";
				addNamespace(fPrefix,ns);
				freeze();		
				break;
			}
			case XSCValue.NAME_VALUE:
			{
				fPrefix = ((XSCNameValue)value).getXMLString();
				break;
			}
			case XSCValue.ANNOT_VALUE:
			{ XSCDOMBuilder.getBuilder().getSchema().property(value,0); break; }
			default:
				throw new XSCException(XSCException.IllegalProperty,value);
		}
	}

	protected void freeze_() throws XSCException
	{}

}
