package noown.domain.xsc.comp;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCNameValue;
import noown.domain.xsc.val.XSCStringValue;
import noown.domain.xsc.val.XSCURIValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Notation component
 * 
 * @author Kilian Stillhard
 */
public class XSCNotation extends XSCComponent
{
	private XSCNameValue name;
	private XSCStringValue pub;
	private XSCURIValue sys;
	private XSCAnnotationValue annot;
	
	public XSCNotation() throws XSCException
	{
		super();
	}

	public XSCNotation(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.NOTATION;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.NAME_VALUE:
			{ name = (XSCNameValue)value; break; }
			case XSCValue.STRING_VALUE:
			{ pub = (XSCStringValue)value; break; }
			case XSCValue.URI_VALUE:
			{ sys = (XSCURIValue)value; break; }
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value);break;}
			default: throw new XSCException(XSCException.IllegalProperty,value);
		}
	}

	protected void freeze_() throws XSCException
	{
		Element no = XSCDOMBuilder.getBuilder().createSchemaElement("notation");
		XSCCommon.addAnnotation(no,annot);
		no.setAttribute("name",name.getXMLString());
		no.setAttribute("public",pub.getXMLString());
		no.setAttribute("system",sys.getXMLString());
		((XSCNodeListImpl)fNodes).add(no);
	}

}
