package noown.domain.xsc.comp;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCPatternValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Pattern facet
 * 
 * @author Kilian Stillhard
 */
public class XSCPatternFacet extends XSCFacet
{

	private XSCPatternValue pattern;
	private XSCAnnotationValue annot;

	public XSCPatternFacet() throws XSCException
	{
		super();
	}

	public XSCPatternFacet(NodeList nodes) throws XSCException
	{
		super(nodes);
	}
	
	public int getType()
	{
		return XSCValue.PATTERN_FACET;
	}
	
	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		super.property_(value,subtype);
		switch (value.getType())
		{
			case XSCValue.PATTERN_VALUE:
			{ pattern = (XSCPatternValue)value; break; }
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value);break;}
			default: break;
		}
	}

	protected void freeze_() throws XSCException
	{
		Element pat = XSCDOMBuilder.getBuilder().createSchemaElement("pattern");
		XSCCommon.addAnnotation(pat,annot);
		pat.setAttribute("value",pattern.getXMLString());
		
		if (isFixed() || isFixedMin() || isFixedMax())
			throw new XSCException(XSCException.FixedPattern);
			
		((XSCNodeListImpl)fNodes).add(pat);
	}

}
