package noown.domain.xsc.comp;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCRangeValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Range facet
 * 
 * @author Kilian Stillhard
 */
public class XSCRangeFacet extends XSCFacet
{
	private XSCRangeValue range;
	private XSCAnnotationValue annot;
	
	public XSCRangeFacet() throws XSCException
	{
		super();
	}

	public XSCRangeFacet(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.RANGE_FACET;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		super.property_(value,subtype);
		switch (value.getType())
		{
			case XSCValue.RANGE_VALUE:
			{ range = (XSCRangeValue)value; break; }
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value);break;}
			default: break;
		}
	}

	protected void freeze_() throws XSCException
	{
		String min = range.getMin();
		String max = range.getMax();
		
		Element last = null;
		
		if (min != null)
		{
			Element minEl;
			if (range.getMinIncl())
				minEl = XSCDOMBuilder.getBuilder().createSchemaElement("minInclusive");
			else
				minEl = XSCDOMBuilder.getBuilder().createSchemaElement("minExclusive");
			last = minEl;
			minEl.setAttribute("value",min);
			if (isFixed() || isFixedMin()) minEl.setAttribute("fixed","true");
			((XSCNodeListImpl)fNodes).add(minEl);
		}
		if (max != null)
		{
			Element maxEl;
			if (range.getMaxIncl())
				maxEl = XSCDOMBuilder.getBuilder().createSchemaElement("maxInclusive");
			else
				maxEl = XSCDOMBuilder.getBuilder().createSchemaElement("maxExclusive");
			last = maxEl;
			maxEl.setAttribute("value",max);
			if (isFixed() || isFixedMax()) maxEl.setAttribute("fixed","true");
			((XSCNodeListImpl)fNodes).add(maxEl);
		}
		if (last != null) XSCCommon.addAnnotation(last,annot);
	}

}
