package noown.domain.xsc.comp;

import java.util.Vector;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCURIValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Redefine statement
 * 
 * @author Kilian Stillhard
 */
public class XSCRedefine extends XSCComponent
{
	private XSCURIValue fURI;
	private Vector fInner;
	private XSCAnnotationValue annot;

	public XSCRedefine() throws XSCException
	{
		super();
		fInner = new Vector();
	}

	public XSCRedefine(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.REDEFINE;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.URI_VALUE:
			{
				fURI = (XSCURIValue)value;
				break;
			}
			case XSCValue.SIMPLE_TYPE:
			case XSCValue.COMPLEX_TYPE:
			case XSCValue.GROUP:
			case XSCValue.ATTRIBUTE_GROUP:
			{ fInner.addElement((XSCComponent)value); break; }
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value); break;}
			default:
				throw new XSCException(XSCException.IllegalProperty,value);
		}
	}

	protected void freeze_() throws XSCException
	{
		Element red = XSCDOMBuilder.getBuilder().createSchemaElement("redefine");
		XSCCommon.addAnnotation(red,annot);
		red.setAttribute("schemaLocation",fURI.getXMLString());
		for (int i = 0; i < fInner.size();i++)
		{
			addNodes(red,((XSCComponent)fInner.elementAt(i)).getNodes());
		}
		
		((XSCNodeListImpl)fNodes).add(red);
	}

}
