package noown.domain.xsc.comp;

import java.util.Vector;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCNameValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Restriction component
 * 
 * @author Kilian Stillhard
 */
public class XSCRestriction extends XSCComponent
{
	
	private XSCNameValue base;
	private XSCComponent innerST;
	private Vector facets;
	private XSCAnnotationValue annot;
	private boolean hasFacets;

	public XSCRestriction() throws XSCException
	{
		super();
		facets = new Vector();
	}

	public XSCRestriction(NodeList nodes) throws XSCException
	{
		super(nodes);
	}
	
	public boolean isTypeRef()
	{
		return ((base != null) && (!hasFacets));
	}
	
	public XSCNameValue getTypeRef()
	{
		return base;
	}

	public int getType()
	{
		return XSCValue.RESTRICTION;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.NAME_VALUE:
			{ base = (XSCNameValue)value; break; }
			
			case XSCValue.KEYWORD_VALUE:
			{
				if (((XSCKeywordValue)value).getKey() == XSCParserConstants.OPEN_PAR)
					hasFacets = true;
				break;
			}
			case XSCValue.RESTRICTION:
			case XSCValue.UNION:
			case XSCValue.LIST:
			{ innerST = (XSCComponent)value; break; }
			
			case XSCValue.RANGE_FACET:
			case XSCValue.LENGTH_FACET:
			case XSCValue.ENUM_FACET:
			case XSCValue.PATTERN_FACET:
			case XSCValue.TOTDIG_FACET:
			case XSCValue.FRACDIG_FACET:
			case XSCValue.WHITESP_FACET:
			{ facets.add(value); break; }
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value); break;}
			
			default:
				throw new XSCException(XSCException.IllegalProperty,value);
		}
	}

	protected void freeze_() throws XSCException
	{
		Element el = XSCDOMBuilder.getBuilder().createSchemaElement("restriction");
		XSCCommon.addAnnotation(el,annot);						
		if (base != null)
			el.setAttribute("base",base.getXMLString());
			
		if (innerST != null)
		{
			XSCSimpleType st = new XSCSimpleType();
			st.property(innerST,0);
			st.freeze();
			addNodes(el,st.getNodes());
		}
		
		for (int i = 0; i < facets.size(); i++)
			addNodes(el,((XSCComponent)facets.elementAt(i)).getNodes());
			
		((XSCNodeListImpl)fNodes).add(el);
	}

}
