package noown.domain.xsc.comp;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.XSCWarning;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Compact schema main component
 * 
 * @author Kilian Stillhard
 */
public class XSCSchema extends XSCComponent
{
	private boolean hadDefault;
	private boolean hadDefaultEl;
	private boolean hadDefaultAtt;
	private boolean hadVersion;
	
	
	public XSCSchema() throws XSCException
	{
		super();
		hadDefault = false;
		hadDefaultEl = false;
		hadDefaultAtt = false;
		hadVersion = false;
	}
	

	public XSCSchema(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.SCHEMA;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.TARGET_NAMESPACE:
			case XSCValue.NAMESPACE:
				break;
		
			case XSCValue.VERSION:
			{
				if (hadVersion) throw new XSCException(XSCException.DupVersion);
				hadVersion = true;
			}
			case XSCValue.DEFAULT:
			{
				if (hadDefault) throw new XSCException(XSCException.DupDefault);
				hadDefault = true;
			}
			case XSCValue.DEFAULT_EL:
			{
				if (hadDefaultEl) throw new XSCException(XSCException.DupElDefault);
				hadDefaultEl = true;
			}
			case XSCValue.DEFAULT_ATTR:
			{
				if (hadDefaultAtt) throw new XSCException(XSCException.DupAttDefault);
				hadDefaultAtt = true;
			}
			case XSCValue.IMPORT:
			case XSCValue.INCLUDE:
			case XSCValue.REDEFINE:
			
			case XSCValue.ELEMENT:
			case XSCValue.ATTRIBUTE:
			case XSCValue.COMPLEX_TYPE:
			case XSCValue.SIMPLE_TYPE:
			case XSCValue.GROUP:
			case XSCValue.ATTRIBUTE_GROUP:
			case XSCValue.NOTATION:
			{
				addNodes(XSCDOMBuilder.getBuilder().getDoc().getDocumentElement(),
							((XSCComponent)value).getNodes());
				break;
			}

			case XSCValue.ANNOT_VALUE:
			{ 
				// special treatment for annotations, they have to be added where
				// they occur, not as the first child element
				XSCCommon.addAnnotationApp(XSCDOMBuilder.getBuilder().getDoc().getDocumentElement(),
										(XSCAnnotationValue)value);
			 	break;
			}
			
			default:
				throw new XSCException(XSCException.IllegalProperty,value);
		}
	}

	protected void freeze_() throws XSCException
	{
		// check if there was a element default statement
		if (!hadDefaultEl)
		{
			XSCElementDefault elDef = new XSCElementDefault();
			elDef.property(new XSCKeywordValue(XSCParserConstants.QUALIFIED),0);
			elDef.freeze();
			addNodes(XSCDOMBuilder.getBuilder().getDoc().getDocumentElement(),elDef.getNodes());
		}
		
		// namespace cleanup
		XSCNamespace.cleanup();
		
		// set the right prefix for the schema element
		String schemaPrefix = XSCNamespace.getSchemaPrefix();
		XSCDOMBuilder.getBuilder().getDoc().getDocumentElement().setPrefix(schemaPrefix);
		
		addNodes(XSCDOMBuilder.getBuilder().getDoc().getDocumentElement(),XSCNamespace.getNodesNS());
	}

}
