package noown.domain.xsc.comp;

import java.util.Vector;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCNameValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Simple type component
 * 
 * @author Kilian Stillhard
 */
public class XSCSimpleType extends XSCComponent
{
	
	private XSCNameValue name;
	private XSCComponent typeDef;
	private Vector options;
	private XSCAnnotationValue annot;

	public XSCSimpleType() throws XSCException
	{
		super();
		options = new Vector();
	}

	public XSCSimpleType(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.SIMPLE_TYPE;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.NAME_VALUE:
			{ name = (XSCNameValue)value; break; }
			
			case XSCValue.KEYWORD_VALUE:
			{
				switch (((XSCKeywordValue)value).getKey())
				{
					case XSCParserConstants.FINAL:
					case XSCParserConstants.FINAL_EXT:
					case XSCParserConstants.FINAL_RESTR:
					case XSCParserConstants.FINAL_LIST:
					case XSCParserConstants.FINAL_UNION:
					{ options.add(value); break; }
					default:
						throw new XSCException(XSCException.IllQSimpleType,value);
				}
				break;
			}
			
			case XSCValue.RESTRICTION:
			case XSCValue.LIST:
			case XSCValue.UNION:
			{
				typeDef = (XSCComponent)value;
				break;
			}
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value);break;}
			default:
				throw new XSCException(XSCException.IllegalProperty,value);
		}
	}

	protected void freeze_() throws XSCException
	{
		Element el = XSCDOMBuilder.getBuilder().createSchemaElement("simpleType");
		XSCCommon.addAnnotation(el,annot);						
		if (name != null)
			el.setAttribute("name",name.getXMLString());
			
		String fin = XSCCommon.getFinalAttr(options);
		if (fin != null) el.setAttribute("final",fin);
		
		if (typeDef != null)
			addNodes(el,typeDef.getNodes());
			
		((XSCNodeListImpl)fNodes).add(el);
	}

}
