package noown.domain.xsc.comp;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCIntValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Total digits facet
 * 
 * @author Kilian Stillhard
 */
public class XSCTotalDigitsFacet extends XSCFacet
{
	private XSCIntValue val;
	private XSCAnnotationValue annot;

	public XSCTotalDigitsFacet() throws XSCException
	{
		super();
	}

	public XSCTotalDigitsFacet(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.TOTDIG_FACET;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		super.property_(value,subtype);
		switch (value.getType())
		{
			case XSCValue.INTEGER_VALUE:
			{ val = (XSCIntValue)value; break; }
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value);break;}
			default: break;
		}
	}

	protected void freeze_() throws XSCException
	{
		Element td = XSCDOMBuilder.getBuilder().createSchemaElement("totalDigits");
		XSCCommon.addAnnotation(td,annot);
		td.setAttribute("value",val.getXMLString());
		if (isFixed()) td.setAttribute("fixed","true");
		if (isFixedMin() || isFixedMax())
			throw new XSCException(XSCException.FixedTotDig);
		((XSCNodeListImpl)fNodes).add(td);
	}
}
