package noown.domain.xsc.comp;

import java.util.Vector;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Union simple type definition
 * 
 * @author Kilian Stillhard
 */
public class XSCUnion extends XSCComponent
{
	private Vector members;
	private XSCAnnotationValue annot;

	public XSCUnion() throws XSCException
	{
		super();
		members = new Vector();
	}

	public XSCUnion(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.UNION;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.RESTRICTION:
			case XSCValue.UNION:
			case XSCValue.LIST:
			{ members.add(value); break; }
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value);break;}
			default:
				throw new XSCException(XSCException.IllegalProperty);
		} 
	}

	protected void freeze_() throws XSCException
	{
		Element un = XSCDOMBuilder.getBuilder().createSchemaElement("union");
		XSCCommon.addAnnotation(un,annot);
		StringBuffer memberTypes = new StringBuffer();
		boolean hadMemberTypes = false;
		
		for (int i = 0; i < members.size(); i++)
		{
			if (((XSCComponent)members.elementAt(i)).getType() == XSCValue.RESTRICTION &&
				 ((XSCRestriction)members.elementAt(i)).isTypeRef())
			{
				if (hadMemberTypes) memberTypes.append(" ");
				hadMemberTypes = true;
				memberTypes.append(((XSCRestriction)members.elementAt(i)).getTypeRef().getXMLString());
			} else {
				XSCSimpleType st = new XSCSimpleType();
				st.property((XSCComponent)members.elementAt(i),0);
				st.freeze();
				addNodes(un,st.getNodes());
			}
		}
		if (hadMemberTypes)
			un.setAttribute("memberTypes",memberTypes.toString());
			
		((XSCNodeListImpl)fNodes).add(un);
	}

}
