package noown.domain.xsc.comp;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Whitespace facet
 * 
 * @author Kilian Stillhard
 */
public class XSCWhiteSpaceFacet extends XSCFacet
{
	private XSCKeywordValue ws;
	private XSCAnnotationValue annot;
	
	public XSCWhiteSpaceFacet() throws XSCException
	{
		super();
	}

	public XSCWhiteSpaceFacet(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	public int getType()
	{
		return XSCValue.WHITESP_FACET;
	}

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		super.property_(value,subtype);
		switch (value.getType())
		{
			case XSCValue.KEYWORD_VALUE:
			{ 
				switch (((XSCKeywordValue)value).getKey())
				{
					case XSCParserConstants.COLLAPSE:
					case XSCParserConstants.PRESERVE:
					case XSCParserConstants.REPLACE:
					{ ws = (XSCKeywordValue)value; break; }
					default: break;
				}
				break;
			}
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value);break;}
			default: break;
		}
	}

	protected void freeze_() throws XSCException
	{
		Element wsEl = XSCDOMBuilder.getBuilder().createSchemaElement("whiteSpace");
		XSCCommon.addAnnotation(wsEl,annot);	
		wsEl.setAttribute("value",ws.getXMLString());
		if (isFixed()) wsEl.setAttribute("fixed","true");
		if (isFixedMin() || isFixedMax())
			throw new XSCException(XSCException.FixedWS);
		((XSCNodeListImpl)fNodes).add(wsEl);
	}
}
