package noown.domain.xsc.comp;

import java.util.StringTokenizer;
import java.util.Vector;

import noown.domain.xsc.XSCDOMBuilder;
import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCHandler;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.util.XSCNodeListImpl;
import noown.domain.xsc.val.XSCAnnotationValue;
import noown.domain.xsc.val.XSCKeywordValue;
import noown.domain.xsc.val.XSCSimpleValue;
import noown.domain.xsc.val.XSCURIValue;
import noown.domain.xsc.val.XSCValue;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Abstract superclass for the wildcard components
 * 
 * @author Kilian Stillhard
 */
public abstract class XSCWildcard extends XSCComponent
{
	private XSCKeywordValue process;
	private Vector namespaces;
	protected XSCAnnotationValue annot;

	public XSCWildcard() throws XSCException
	{
		super();
		namespaces = new Vector();
	}

	public XSCWildcard(NodeList nodes) throws XSCException
	{
		super(nodes);
	}

	abstract public int getType();

	protected void property_(XSCValue value, int subtype) throws XSCException
	{
		switch (value.getType())
		{
			case XSCValue.KEYWORD_VALUE:
			{
				switch (((XSCKeywordValue)value).getKey())
				{
					case XSCParserConstants.LAX:
					case XSCParserConstants.STRICT:
					case XSCParserConstants._SKIP:
					{ process = (XSCKeywordValue)value; break; }
					
					case XSCParserConstants.OTHER_CONST:
					{
						if (namespaces.size() > 0)
							throw new XSCException(XSCException.OtherInNS);
						else
							namespaces.add(value);
						break;
					}
					case XSCParserConstants.LOCAL_CONST:
					case XSCParserConstants.TARGET_NS_CONST:
					{ 
						if (namespaces.size() > 0 && 
						    (((XSCValue)namespaces.elementAt(0)).getType() == XSCValue.KEYWORD_VALUE) &&
						    (((XSCKeywordValue)namespaces.elementAt(0)).getKey() == XSCParserConstants.OTHER_CONST))
						    throw new XSCException(XSCException.OtherInNS);
						namespaces.add(value);
						break;
					}
					default: break;
				}
				break;
			}
			case XSCValue.URI_VALUE:
			{ 
				if (namespaces.size() > 0 && 
					(((XSCValue)namespaces.elementAt(0)).getType() == XSCValue.KEYWORD_VALUE) &&
				    (((XSCKeywordValue)namespaces.elementAt(0)).getKey() == XSCParserConstants.OTHER_CONST))
				    throw new XSCException(XSCException.OtherInNS);
				namespaces.add(value);
				break;
			}
			case XSCValue.ANNOT_VALUE:
			{ annot = XSCCommon.mergeAnnotations(annot,(XSCAnnotationValue)value); break;}
			default:break;
		}
	}

	abstract protected void freeze_() throws XSCException;
	
	protected NodeList getWildcardAttrs() throws XSCException
	{
		XSCNodeListImpl nodes = new XSCNodeListImpl();
		
		if (process != null)
		{
			Attr pr = XSCDOMBuilder.getBuilder().createSchemaAttr("process");
			pr.setValue(process.getXMLString());
			nodes.add(pr);
		}
		
		if (namespaces.size() > 0)
		{
			Attr ns = XSCDOMBuilder.getBuilder().createSchemaAttr("namespace");
			StringBuffer nsStr = new StringBuffer();
			for (int i = 0; i < namespaces.size(); i++)
			{
				if (i > 0) nsStr.append(" ");
				nsStr.append(((XSCSimpleValue)namespaces.elementAt(i)).getXMLString());
			}
			ns.setValue(nsStr.toString());
			nodes.add(ns);
		}
		return nodes;
	}
}
