package noown.domain.xsc.val;

import java.util.Hashtable;

import noown.domain.xsc.XSCException;
import noown.domain.xsc.XSCParserConstants;
import noown.domain.xsc.XSCWarning;
import noown.domain.xsc.comp.XSCNamespace;
import noown.domain.xsc.util.XSCNodeListImpl;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/**
 * Class that holds a name (QName or NCName).
 * 
 * This class checks whether a given QName is legal, i.e. whether
 * its prefix has been declared earlier using class XSCNamespace.
 * 
 * @author Kilian Stillhard
 */
public class XSCNameValue extends XSCSimpleValue
{
	protected String fName;
	protected static Hashtable fKeywords;
	
	static 
	{
		fKeywords = new Hashtable();
		for (int i = 0; i < XSCParserConstants.tokenImage.length; i++)
		{
			String tok = XSCParserConstants.tokenImage[i];
			if (tok.charAt(0) == '"' && 
				(Character.isLetter(tok.charAt(1)) || tok.charAt(1) == '#'))
				fKeywords.put(tok.substring(1,tok.length()-1),new Boolean(true));
		}
	}
	
	public XSCNameValue(String name) throws XSCException
	{
		fName = name;
		int ind;
		// check if it is a QName
		if ((ind = fName.indexOf(":")) >= 0)
		{
			if (! XSCNamespace.hasPrefix(fName.substring(0,ind)))
				XSCWarning.add("Prefix " + fName.substring(0,ind) + " unknown");
		}
	}
	
	public String getXSCString() throws XSCException
	{
		if (fKeywords.containsKey(fName))
			return '\\' + fName;
		else
			return fName;
	}
	
	public String getXMLString() throws XSCException
	{
		return fName;
	}

	public int getType()
	{
		return XSCValue.NAME_VALUE;
	}
	
	public boolean equals(XSCNameValue n)
	{
		return fName.equals(n.fName);
	}

}
