package noown.domain.xsc.val;

import noown.domain.xsc.XSCException;
import noown.domain.xsc.util.XSCNodeListImpl;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Attr;

/**
 * Class that holds an occurrence specifier value.
 * 
 * This is a special case of a positive integer range.  Note that the value
 * '-1' is used to indicate "unbounded".
 * 
 * @author Kilian Stillhard
 */
public class XSCOccSpecValue extends XSCPosIntRange
{
	
	public XSCOccSpecValue(String minStr, String maxStr)
		throws XSCException
	{
		super(-1,-1);
		try {
			if (minStr != null && minStr.length() > 0)
				fMin = Integer.parseInt(minStr);
			else
				fMin = 1;
		} catch (Exception e) 
		{ throw new XSCException("minStr parse error in XSCOccSpecValue()!",e); }
		
		try {
			if (maxStr != null && maxStr.length() > 0)
				fMax = Integer.parseInt(maxStr);
			else
				fMax = 1;
		} catch (Exception e)
		{
			if (maxStr.indexOf("unbounded") >= 0)
				fMax = -1;
			else
				throw new XSCException("maxStr parse error in XSCOccSpecValue()!",e);
		}
	}
	
	public XSCOccSpecValue(XSCPosIntRange range) throws XSCException
	{
		super(range.fMin, range.fMax);
	}
	
	public XSCOccSpecValue(int min, int max) throws XSCException
	{
		super(min,max);
	}

	public int getType()
	{
		return XSCValue.OCCSPEC_VALUE;
	}
	
	public String getMinString()
	{
		if (fMin != 1 && fMin >= 0)
			return Integer.toString(fMin);
		else
			return null;
	}
	
	public String getMaxString()
	{
		if (fMax == -1) return "unbounded";
		else  return Integer.toString(fMax);

	}

	public String getXMLString() throws XSCException
	{
		throw new XSCException("getXMLString() undefined for XSCOccSpecValue!");
	}

	public String getXSCString() throws XSCException
	{
		if (fMin == 0 && fMax == 1)
			return "?";
		if (fMin == 0 && fMax == -1)
			return "*";
		if (fMin == 1 && fMax == -1)
			return "+";
		String occSpec = "[";
		if (fMin >= 0) occSpec += fMin;
		occSpec += ",";
		if (fMax >= 0) occSpec += fMax;
		occSpec += "]";
		return occSpec;
		
	}

}
