package noown.domain.xsc.val;

import noown.domain.xsc.XSCException;
import org.w3c.dom.NodeList;

/**
 * Class that holds a pattern value (a regular expression).
 * 
 * @author Kilian Stillhard
 */
public class XSCPatternValue extends XSCSimpleValue
{

	protected String fPattern;
	
	public XSCPatternValue(String pattern,boolean escape) throws XSCException
	{
		if (escape)
			fPattern = unescapePattern(pattern);
		else
			fPattern = pattern;
	}

	public int getType()
	{
		return XSCValue.PATTERN_VALUE;
	}

	public String getXMLString() throws XSCException
	{
		return fPattern;
	}

	public String getXSCString() throws XSCException
	{
		return escapePattern(fPattern);
	}
	
	private String unescapePattern(String pattern) throws XSCException
	{
		StringBuffer buf = new StringBuffer(pattern.length());
		
		for (int i = 0; i < pattern.length(); i++)
		{
			switch (pattern.charAt(i))
			{
				case '/': break;
				case '\\':
				{
					if (++i < pattern.length())
						if (pattern.charAt(i) == '/')
							buf.append('/');
						else
						{
							buf.append(pattern.charAt(i-1));
							buf.append(pattern.charAt(i));
						}
					else
						throw new XSCException("Illegal backslash at end of pattern!");
					break;
				}
				default:
					buf.append(pattern.charAt(i));
			}
		}
		
		return buf.toString();
	}
	
	private String escapePattern(String pattern)
	{
		StringBuffer buf = new StringBuffer(pattern.length() + 4);
		
		buf.append('/');
		
		for (int i = 0; i < pattern.length(); i++)
		{
			if (pattern.charAt(i) == '/')
				buf.append("\\/");
			else
				buf.append(pattern.charAt(i));
		}
		
		buf.append('/');
	
		return buf.toString();
	}
}
