package noown.domain.xsc.val;

import noown.domain.xsc.XSCException;
import org.w3c.dom.NodeList;

/**
 * Class that can hold a range of positive and possibly undefined
 * integer values.
 * 
 * @author Kilian Stillhard
 */
public class XSCPosIntRange extends XSCSimpleValue
{
	protected int fMin;
	protected int fMax;
	
	public XSCPosIntRange(int min, int max) throws XSCException
	{
		if (min < -1 || max < -2) throw new XSCException(XSCException.IllIntRangeValue);
		fMin = min;
		fMax = max;
	}

	public int getType()
	{
		return XSCValue.INTRANGE_VALUE;
	}
	
	public int getMin()
	{
		return fMin;
	}
	
	public int getMax()
	{
		return fMax;
	}

	public String getXMLString() throws XSCException
	{
		throw new XSCException("getXMLString() not supported for XSCPosIntRange!");
	}

	public String getXSCString() throws XSCException
	{
		StringBuffer buf = new StringBuffer();
		buf.append('[');
		if (fMin >= 0 ) buf.append(Integer.toString(fMin));
		buf.append(',');
		if (fMax >= 0 ) buf.append(Integer.toString(fMax));
		buf.append(']');
		
		return buf.toString();
	}

}
