package noown.domain.xsc.val;

import noown.domain.xsc.XSCException;
import org.w3c.dom.NodeList;

/**
 * A class that holds a range value. Ranges can have exclusive or
 * inclusive borders and they can hold a variety of numeric datatypes.
 * ( All the datatypes to which the minExclusive etc. facets are applicable)
 * 
 * @author Kilian Stillhard
 */
public class XSCRangeValue extends XSCSimpleValue
{
	protected String fMin;
	protected boolean fMinIncl;
	protected String fMax;
	protected boolean fMaxIncl;
	
	public XSCRangeValue(String min, boolean minIncl, String max, boolean maxIncl)
	{
		fMin = min;
		fMinIncl = minIncl;
		fMax = max;
		fMaxIncl = maxIncl;
	}

	public int getType()
	{
		return XSCValue.RANGE_VALUE;
	}
	
	public String getMin()
	{
		return fMin;
	}
	
	public String getMax()
	{
		return fMax;
	}
	
	public boolean getMinIncl()
	{
		return fMinIncl;
	}
	
	public boolean getMaxIncl()
	{
		return fMaxIncl;
	}

	public String getXMLString() throws XSCException
	{
		throw new XSCException("getXMLString() undefined for XSCRangeValue!");
	}

	public String getXSCString() throws XSCException
	{
		StringBuffer buf = new StringBuffer();
		
		if (fMinIncl)
			buf.append('[');
		else
			buf.append('(');
			
		if (fMin != null) buf.append(fMin);
		buf.append("..");
		if (fMax != null) buf.append(fMax);
		
		if (fMaxIncl)
			buf.append(']');
		else
			buf.append(')');
			
		return buf.toString();
	}

}
