package noown.domain.xsc.val;

import noown.domain.xsc.XSCException;
import org.w3c.dom.NodeList;

/**
 * Class that holds a string value.
 * 
 * Escaping and unescaping of special characters is done here.
 * 
 * @author Kilian Stillhard
 */
public class XSCStringValue extends XSCSimpleValue
{
	protected String fStr;
	
	public XSCStringValue(String str, boolean escape) throws XSCException
	{
		if (escape)
			fStr = unescapeString(str);
		else
			fStr = str;
	}

	public int getType()
	{
		return XSCValue.STRING_VALUE;
	}

	public String getXMLString() throws XSCException
	{
		return fStr;
	}

	public String getXSCString() throws XSCException
	{
		return escapeString(fStr);
	}
	
	protected String unescapeString(String str) throws XSCException
	{
		StringBuffer buf = new StringBuffer(str.length());
		
		for (int i = 0; i < str.length(); i++)
		{
			switch (str.charAt(i))
			{
				case '"': break;
				case '\\':
				{
					if (++i < str.length())
					{
						switch (str.charAt(i))
						{
							case '\\':
							case '"':
							{ buf.append(str.charAt(i)); break; }
							case 'n':
							{ buf.append('\n'); break; }
							case 'r':
							{ buf.append('\r'); break; }
							case 'f':
							{ buf.append('\f'); break; }
							case 't':
							{ buf.append('\t'); break; }
							default:
								throw new XSCException("Illegal character escape in string: " + str);
						}
					} else {
						throw new XSCException("Illegal backslash at end of string: " + str);
					}
					break;
				}
				default:
					buf.append(str.charAt(i)); 
			}
		}
		
		return buf.toString();
	}

	protected String escapeString(String str)
	{
		StringBuffer buf = new StringBuffer(str.length() + 4);
		
		buf.append('"');
		
		for (int i = 0; i < str.length();i++)
		{
			switch (str.charAt(i))
			{
				case '"': {buf.append("\\\""); break; }
				case '\\': {buf.append("\\\\"); break; }
				case '\n': {buf.append("\\n"); break; }
				case '\r': {buf.append("\\r"); break; }
				case '\f': {buf.append("\\f"); break; }
				default: {buf.append(str.charAt(i)); }
			}
		}
		
		buf.append('"');
		
		return buf.toString();
	}


}
